/** 
 * CISC422/853: ASSIGNMENT 4, March 2009
 * Instructor: Juergen Dingel
 * File: main.java
 * 
 * Purpose 
 * 		The purpose of package imp is to implement a parser and
 * 		backward slicer for a simple, imperative language IMP.
 * IMP:
 * 		In the following, let "d" denote the directory that this 
 * 		file is in. The grammar of the language IMP is described in 
 * 		"d/impGrammar.txt" and is implemented in "d/parser/imp1.cup".
 * 		LIMP is a proper subset of a simple imperative language IMP.
 * 		IMP supports assignments, print, skip, branching (i.e., 
 * 		if-then, if-then-else), and looping (i.e., while and repeat). 
 * Parser: 
 * 		The parser supports all of IMP. It is in "d/parser/parse.java" 
 * 		and has been generated automatically using the "java_cup" 
 * 		parser generator. The file "d/parser/imp1.cup" contains the 
 * 		description of IMP for "java_cup". 
 * Usage:
 *		From the commandline from the directory above "d" do, e.g., 
 *			"java imp.Main imp/programs/p1.imp x y z"
 *		where "imp/programs/p1.imp" is the path of the file
 *		containing the IMP program to be sliced and "x", "y", and "z"
 *		are the criterion variables. The criterion location
 *		always is the last node in the program (i.e., the final
 *		"END" node).
 * To do for the assignment: 
 * 		Implement a static backwards slicer for IMP. 
 * 		Currently, the code should compile and run. It should display 
 * 		the original program and the slicing criteria correctly. 
 * 		However, the slice will always consist of just the first 
 * 		and the last node of the abstract syntax tree. To make the 
 * 		code compute the slice correctly, do the following:
 * 		1) A node has a "dRVars" attribute used to store the directly 
 * 		   relevant variables. This attribute has type "VarIdSet". 
 * 		   To query and manipulate "dRVars", you have to complete 
 * 		   the implementation of "VarIdSet" appropriately.
 * 		2) Complete the code for the method "computeDRVars"
 * 		   for all nodes. For each node (except "ProgramEndNode") 
 * 		   "computeDRVars" should:
 * 		   a) compute the directly relevant variables ("dRVars") of the node, 
 * 		   b) determine if the node is relevant and if so, mark it as such by
 * 			  setting the boolean attribute isRelevant,
 * 		   c) call "computeDRVars" on all predecessors.
 * 		   For "ProgramEndNode", b) is not necessary, because "toStringUsingRelevant"
 * 		   in "Cfg.java" will always print the "ProgramBeginNode" and the 
 * 		   "ProgramEndNode".
 */
package imp;

import java.lang.Exception;
import java.lang.reflect.Array;

import imp.parser.Program;
import imp.parser.Parse;
import imp.parser.ProgramBody;
import imp.slicer.Cfg;
import imp.slicer.Node;
import imp.util.VarIdSet;

class Main {
	/**
	 * This method is responsible for loading an IMP program and
	 * displaying its completed slice based on the user's input,
	 * which is the following arguments:
	 * - First argument: the (relative) path of the original file
	 * - All other arguments: the relevant variables
	 * The last node in the program will always be used as the criterion 
	 * location. The original program, slicing criteria, and the 
	 * resulting slice are all written to STDOUT. 
	 */ 
    public static void main(String arguments[]) throws Exception {
    	String programFilePath = null;
    	Program program = null;
    
    	int argumentsLength = Array.getLength(arguments);
    	if (argumentsLength < 2) {
    		System.out.println("*** ERROR: Must provide at least 2 arguments: " + "file name and criterion variable(s)" + "\n*** TERMINATING ***");
    		System.exit(1);
    	}
	
    	// read in command line arguments
    	programFilePath = arguments[0];
    	VarIdSet cVars = new VarIdSet();
    	for (int i = 1; i < argumentsLength; i++) {
    		cVars.add(arguments[i]);
    	}
		
    	// parse program
    	try {
    		program = Parse.parse(programFilePath);
    	}
    	catch (Exception e) {
    		System.out.println("\n*** ERROR: Input program not syntactically correct ***\n");
    		System.exit(1);
    	}
    	System.out.println("\n*** ORIGINAL PROGRAM ***\n");
    	System.out.println(program.toString(0));
	
    	System.out.println();
    	// parse the program
    	ProgramBody progBody = program.programBody;
    	Cfg cfg = progBody.toCfg(0);
    	
    	System.out.println("*** SLICING CRITERIA ***\n");
    	// last node is criterion node by default
    	Node cNode = cfg.last;
    	System.out.println("Location: " + cNode.toString());
    	System.out.println("Variables: " + cVars.toString());
    		
    	System.out.println("\n*** SLICED PROGRAM (WITHOUT VARIABLE DECLARATIONS) ***\n");
    	// compute slice
    	cfg.computeSlice(cNode, cVars);
    	
    	// print sliced program
    	System.out.println(cfg.toStringUsingRelevant());	
    }  
    
} // class Main

