package imp.parser;

import imp.slicer.Cfg;
import imp.slicer.IfThenElseTestNode;
import imp.slicer.Node;
import imp.slicer.TestEndNode;
import imp.util.Util;

public class IfThenElseStmt extends IfThenStmt {
    public StmtList elseStmtList; 

    public IfThenElseStmt(Loc from, Loc to, Exp test, StmtList thenStmtList, 
		    StmtList elseStmtList ) {
    	super(from, to, test, thenStmtList);
    	this.test = test; 
    	this.thenStmtList = thenStmtList; 
    	this.elseStmtList = elseStmtList;
    }

    public String toString(int i) {
    	String fromStr = from.toString();
    	String toStr = to.toString();
    	String testStr = test.toString();
    	String thenStmtListStr = thenStmtList.toString(i+1);
    	String indStr = Util.makeIndentStr(i);
    	String elseStmtListStr = elseStmtList.toString(i+1);
    	return fromStr+indStr+"IF "+testStr+" THEN\n"+thenStmtListStr+indStr+"ELSE\n"+elseStmtListStr+toStr+indStr+"END";
    }

    public Cfg toCfg(int indentLevel) {
    	// make cfg of then-branch
    	Cfg tCfg = thenStmtList.toCfg(indentLevel+1);
    	// get its first node
    	Node tFirstNode = tCfg.first;
    	// make cfg of else-branch
    	Cfg eCfg = elseStmtList.toCfg(indentLevel+1);
    	// get its first node
    	Node eFirstNode = eCfg.first;	
    	// create endNode
    	TestEndNode endNode = new TestEndNode(to, indentLevel);
    	// create test node
    	IfThenElseTestNode testNode = 
    		new IfThenElseTestNode(from, test, indentLevel, endNode, tFirstNode, eFirstNode);
    	
    	// connect first node of then-brach and test node 
    	testNode.addNext(tFirstNode);
    	tFirstNode.addPrev(testNode);
    	// connect first node of else-branch and test node
    	testNode.addNext(eFirstNode);
    	eFirstNode.addPrev(testNode);
    	// connect last node of then-branch and end node
    	Node tLastNode = tCfg.last;
    	endNode.addPrev(tLastNode);
    	tLastNode.addNext(endNode); 	
    	// connect last node of else-branch and end node
    	Node eLastNode = eCfg.last;
    	endNode.addPrev(eLastNode);
    	eLastNode.addNext(endNode);
    	
    	return new Cfg(testNode, endNode);
    }
    
} // public class CondStmt extends Stmt 
