package imp.parser;

import imp.slicer.Cfg;
import imp.slicer.IfThenTestNode;
import imp.slicer.Node;
import imp.slicer.TestEndNode;
import imp.util.Util;

public class IfThenStmt extends Stmt {
    public Exp test; 
    public StmtList thenStmtList; 

    public IfThenStmt(Loc from, Loc to, Exp test, StmtList thenStmtList) {
    	super(from, to);
    	this.test = test; 
    	this.thenStmtList = thenStmtList; 
    }
    
    public String toString(int i) {
    	String fromStr = from.toString();
    	String toStr = to.toString();
    	String testStr = test.toString();
    	String thenStmtListStr = thenStmtList.toString(i+1);
    	String indStr = Util.makeIndentStr(i);
    	return 
    		fromStr+indStr+"IF "+testStr+" THEN\n"+thenStmtListStr+toStr+indStr+"END"; 
    }

    public Cfg toCfg(int indentLevel) {
    	// make cfg of then-branch
    	Cfg tCfg = thenStmtList.toCfg(indentLevel+1);
    	// get its first node
    	Node tFirstNode = tCfg.first;		
    	// create endNode
    	TestEndNode endNode = new TestEndNode(to, indentLevel);
    	// create test node
    	IfThenTestNode testNode = 
    		new IfThenTestNode(from, test, indentLevel, endNode, tFirstNode);
   
    	// connect test node and first node of then-brach
    	testNode.addNext(tFirstNode);
    	tFirstNode.addPrev(testNode);
    	// connect last node of then-branch and end node
    	Node tLastNode = tCfg.last;
    	endNode.addPrev(tLastNode);
    	tLastNode.addNext(endNode); 	
    	// connect test node and end node
    	testNode.addNext(endNode);
    	endNode.addPrev(testNode);
    	
    	return new Cfg(testNode, endNode);
    }
    
} // public class CondStmt extends Stmt 
