package imp.parser;

import imp.slicer.Cfg;
import imp.slicer.Node;
import imp.slicer.ProgramBeginNode;
import imp.slicer.ProgramEndNode;

public class ProgramBody {
    public StmtList stmtList; 
    public Loc from;
    public Loc to;
    
    public ProgramBody(Loc from, Loc to, StmtList stmtList) {
    	this.stmtList = stmtList; 
    	this.from = from;
    	this.to = to;
    }
    
    public String toString(int i) {
    	String fromStr = from.toString();
    	String toStr = to.toString();
    	String stmtListStr = stmtList.toString(i+1);
    	return fromStr + "BEGIN\n" + stmtListStr + toStr + "END\n";
    }
    
    public Cfg toCfg(int indentLevel) {
    	// make ProgramBeginNode
    	ProgramBeginNode programBeginNode = new ProgramBeginNode(from, indentLevel);
    	// make ProgramEndNode
    	ProgramEndNode programEndNode = new ProgramEndNode(to, indentLevel, programBeginNode);
    	// make CFG for statement list
    	Cfg cfg = stmtList.toCfg(indentLevel+1);
    	// get first node
    	Node firstNode = cfg.first;
    	// get last node
    	Node lastNode = cfg.last;  	 
    	// connect ProgramBeginNode and first node
    	programBeginNode.addNext(firstNode);
    	firstNode.addPrev(programBeginNode);
    	// connect ProgramEndNode and last node
    	programEndNode.addPrev(lastNode);
    	lastNode.addNext(programEndNode);
    	return new Cfg(programBeginNode, programEndNode);
    }
    
} // class ProgramBody 

