package imp.parser;

import imp.slicer.Cfg;
import imp.slicer.Node;
import imp.slicer.RepeatNode;
import imp.slicer.RepeatTestNode;
import imp.util.Util;

public class RepeatStmt extends Stmt {
    	public Exp test; 
    	public StmtList stmtList; 
    	public RepeatStmt(Loc from, Loc to, Exp test, StmtList stmtList ) {
    	super(from, to);
		this.test = test; 
		this.stmtList = stmtList; 
    }
    	
    public String toString(int i) {
    	String fromStr = from.toString();
    	String toStr = to.toString();
    	String expStr = test.toString();
    	String stmtListStr = stmtList.toString(i+1);
    	String indStr = Util.makeIndentStr(i);
    	return 
	    	fromStr+indStr+"REPEAT\n"+stmtListStr+toStr+indStr+"UNTIL "+expStr;
    }
    
    public Cfg toCfg(int indentLevel) {
    	// create cfg of body
    	Cfg cfg = stmtList.toCfg(indentLevel+1);
    	// get its first node 
    	Node firstBodyNode = cfg.first;	
    	// create test node	
    	RepeatTestNode testNode = new RepeatTestNode(to, test, indentLevel, firstBodyNode);    	
    	// create Repeat node 
    	RepeatNode repeatNode = new RepeatNode(indentLevel, from, testNode);
    	// connect repeat node to first body node
    	repeatNode.addNext(firstBodyNode);
    	firstBodyNode.addPrev(repeatNode);
	
    	// connect test node and last body node
    	Node lastNode = cfg.last;
    	lastNode.addNext(testNode);
    	testNode.addPrev(lastNode);
    	// connect test node and first body node
    	testNode.addNext(firstBodyNode);
    	firstBodyNode.addPrev(testNode);
	
    	return new Cfg(repeatNode, testNode);
    }
    
} // public class RepeatStmt extends Stmt 
