package imp.parser;

import imp.slicer.Cfg;
import imp.slicer.Node;
import imp.slicer.TestEndNode;
import imp.slicer.WhileTestNode;
import imp.util.Util;

public class WhileStmt extends Stmt {
    public Exp test; 
    public StmtList stmtList; 
    
    public WhileStmt(Loc from, Loc to, Exp test, StmtList stmtList ) {
    	super(from, to);
    	this.test = test; 
    	this.stmtList = stmtList; 
    }
    
    public String toString(int i) {
    	String fromStr = from.toString();
    	String toStr = to.toString();
    	String expStr = test.toString();
    	String stmtListStr = stmtList.toString(i+1);
    	String indStr = Util.makeIndentStr(i);
    	return 
	    	fromStr+indStr+"WHILE "+expStr+" DO\n"+stmtListStr+toStr+indStr+"END";
    }
    
    public Cfg toCfg(int i) {
    	// make cfg of body 
    	Cfg cfg = stmtList.toCfg(i+1);
    	// get its first node
    	Node firstNode = cfg.first;
    	// create end node
    	TestEndNode endNode = new TestEndNode(to, i);
    	// create test node	
    	WhileTestNode testNode = new WhileTestNode(from, test, i, endNode, firstNode);
    	
    	// connect first body node and test node
    	testNode.addNext(firstNode);
    	firstNode.addPrev(testNode);
    	// connect last body node and test node
    	Node lastNode = cfg.last;
    	lastNode.addNext(testNode);
    	testNode.addPrev(lastNode);
    	// connect test node and end node
    	endNode.addPrev(testNode);
    	testNode.addNext(endNode);

    	return new Cfg(testNode, endNode);
    }
    
} // public class WhileStmt extends Stmt 
