// Definition file of Imp for lexer generator
package imp.parser;

import java.io.*;
import java_cup.runtime.Symbol;  // definition of scanner/parser interface

class ImpLex {
    // reads from file f and produces stream of token on System.out
    // invoke with "java imp.parser.ImpLex f"
    public static void main(String argv[]) throws java.io.IOException {
      String fn = argv[0];
      try {
	BufferedReader in = new BufferedReader(new FileReader(fn));
	lexer theLexer = new lexer(in);
	Symbol s = theLexer.next_token();
	while (s.sym != sym.EOF) {
	  System.out.println("Token: "+Utility.symToString(s.sym)+
			     "("+s.sym+") "+s.value);
	  s = theLexer.next_token();
	}
	System.out.println("Token: "+Utility.symToString(s.sym)+
			   "("+s.sym+") "+s.value);
      }
      catch (IOException e) {
	System.out.println("I/O error occurred while reading from file "+fn);
	System.exit(1);
      } // end try/catch
    }
}

/* semantic value of token returned by scanner */
class TokenValue {          
    public String text;
    public int lineBegin;
    public int charBegin;
    public int charEnd;
    public String filename;   

    TokenValue() {
    }

    TokenValue(String text, int lineBegin, int charBegin, int charEnd) {
	this.text = text; 
	this.lineBegin = lineBegin; 
	this.charBegin = charBegin;
	this.charEnd = charEnd;
	this.filename = null;
    }

   // some functions to convert the value
   public String toString() { 
     return text;
   }

   public boolean toBoolean() {
     return Boolean.valueOf(text).booleanValue();  
   }
   
   public int toInt() {
     return Integer.valueOf(text).intValue();
   }
} // TokenValue

/* some useful helper functions */
class Utility {

    static public int findKeyWord(String str) {
      String strUp = str.toUpperCase();
      if (strUp.equals("PROGRAM")) return sym.T_PROG;
      else if (strUp.equals("VAR")) return sym.T_VAR;
      else if (strUp.equals("INT")) return sym.T_TYINT;
      else if (strUp.equals("BOOL")) return sym.T_TYBOOL;
      else if (strUp.equals("BEGIN")) return sym.T_BEGIN;
      else if (strUp.equals("END")) return sym.T_END;
      else if (strUp.equals("IF")) return sym.T_IF;
      else if (strUp.equals("THEN")) return sym.T_THEN;
      else if (strUp.equals("ELSE")) return sym.T_ELSE;
      else if (strUp.equals("WHILE")) return sym.T_WHILE;
      else if (strUp.equals("REPEAT")) return sym.T_REPEAT;
      else if (strUp.equals("UNTIL")) return sym.T_UNTIL;
      else if (strUp.equals("DO")) return sym.T_DO;
      else if (strUp.equals("SKIP")) return sym.T_SKIP;
      else if (strUp.equals("PRINT")) return sym.T_PRINT;
      else if (strUp.equals("FALSE")) return sym.T_BOOL;
      else if (strUp.equals("TRUE")) return sym.T_BOOL;
      else return sym.T_IDENTIFER;
    }

    static public String symToString(int i) {
        switch (i) {
	case sym.T_TYINT : return "T_TYINT";
	case sym.T_WHILE : return "T_WHILE";
	case sym.T_UNTIL : return "T_UNTIL";
	case sym.T_REPEAT : return "T_REPEAT";
	case sym.T_ELSE : return "T_ELSE";
	case sym.T_SKIP : return "T_SKIP";
	case sym.T_PRINT : return "T_PRINT";
	case sym.T_RPAREN : return "T_RPAREN";
	case sym.T_EQ : return "T_EQ";
	case sym.T_THEN : return "T_THEN";
	case sym.T_TYBOOL : return "T_TYBOOL";
	case sym.T_PROG : return "T_PROG";
	case sym.T_INT : return "T_INT";
	case sym.T_LESSEQ : return "T_LESSEQ";
	case sym.T_ASSIGN : return "T_ASSIGN";
	case sym.T_NOT : return "T_NOT";
	case sym.T_AND : return "T_AND";
	case sym.T_DO : return "T_DO";
	case sym.EOF: return "T_EOF";
	case sym.T_EOF: return "T_EOF";
	case sym.T_MINUS : return "T_MINUS";
	case sym.error: return "T_error";
	case sym.T_SEMI: return "T_SEMI";
	case sym.T_LESS : return "T_LESS";
	case sym.T_BEGIN : return "T_BEGIN";
	case sym.T_BOOL : return "T_BOOL";
	case sym.T_IDENTIFER : return "T_IDENTIFER";
	case sym.T_PLUS : return "T_PLUS";
	case sym.T_OR : return "T_OR";
	case sym.T_END : return "T_END";
	case sym.T_DIVIDE : return "T_DIVIDE";
	case sym.T_ERROR: return "T_ERROR";
	case sym.T_IF : return "T_IF";
	case sym.T_TIMES : return "T_TIMES";
	case sym.T_COLON: return "T_COLON";
	case sym.T_VAR : return "T_VAR";
	case sym.T_LPAREN : return "T_LPAREN";
        default: return "unknown token";
        }
    } 

  static private final String errorMsg[] = {
    "Error: Illegal character."
  };

  static public final int E_UNMATCHED = 0; 

  static public void error( int code ) {
      System.out.println(errorMsg[code]);
  }
} // Utility

%%
%class lexer
%cup

// Value returned on EOF should be special value defined by CUP
%eofval{
    return (new Symbol(sym.EOF, yychar, yychar, (new TokenValue(null,yyline,yychar,yychar))));
%eofval}

%{
    public String sourceFilename;
%}
%line
%char

ALPHA=[A-Za-z]
DIGIT=[0-9]
NONNEWLINE_WHITE_SPACE_CHAR=[\ \t\b\012]
WHITE_SPACE_CHAR=[\n\ \t\b\012]

%% 

<YYINITIAL> ":"		{ return (new Symbol(sym.T_COLON, yychar, yychar+1, new TokenValue(yytext(), yyline, yychar, yychar+1))); }
<YYINITIAL> ";"		{ return (new Symbol(sym.T_SEMI, yychar, yychar+1, new TokenValue(yytext(), yyline, yychar, yychar+1))); }
<YYINITIAL> "("		{ return (new Symbol(sym.T_LPAREN, yychar, yychar+1, new TokenValue(yytext(), yyline, yychar, yychar+1))); }
<YYINITIAL> ")"		{ return (new Symbol(sym.T_RPAREN, yychar, yychar+1, new TokenValue(yytext(), yyline, yychar, yychar+1))); }
<YYINITIAL> "+"		{ return (new Symbol(sym.T_PLUS, yychar, yychar+1, new TokenValue(yytext(), yyline, yychar, yychar+1))); }
<YYINITIAL> "-"		{ return (new Symbol(sym.T_MINUS, yychar, yychar+1, new TokenValue(yytext(), yyline, yychar, yychar+1))); }
<YYINITIAL> "*"		{ return (new Symbol(sym.T_TIMES, yychar, yychar+1, new TokenValue(yytext(), yyline, yychar, yychar+1))); }
<YYINITIAL> "/"		{ return (new Symbol(sym.T_DIVIDE, yychar, yychar+1, new TokenValue(yytext(), yyline, yychar, yychar+1))); }
<YYINITIAL> "="		{ return (new Symbol(sym.T_EQ, yychar, yychar+1, new TokenValue(yytext(), yyline, yychar, yychar+1))); }
<YYINITIAL> "<"		{ return (new Symbol(sym.T_LESS, yychar, yychar+1, new TokenValue(yytext(), yyline, yychar, yychar+1))); }
<YYINITIAL> "<="	{ return (new Symbol(sym.T_LESSEQ, yychar, yychar+2, new TokenValue(yytext(), yyline, yychar, yychar+2))); }
<YYINITIAL> "!"		{ return (new Symbol(sym.T_NOT, yychar, yychar+1, new TokenValue(yytext(), yyline, yychar, yychar+1))); }
<YYINITIAL> "&&"        { return (new Symbol(sym.T_AND, yychar, yychar+2, new TokenValue(yytext(), yyline, yychar, yychar+2))); }
<YYINITIAL> "||"	{ return (new Symbol(sym.T_OR, yychar, yychar+2, new TokenValue(yytext(), yyline, yychar, yychar+2))); }
<YYINITIAL> ":="	{ return (new Symbol(sym.T_ASSIGN, yychar, yychar+2, new TokenValue(yytext(), yyline, yychar, yychar+2))); }

<YYINITIAL>{NONNEWLINE_WHITE_SPACE_CHAR}+ {}

<YYINITIAL> \n  {}

<YYINITIAL>  {DIGIT}+ { 
        String s = new String(yytext()); 
	return (new Symbol(sym.T_INT, yychar, yychar+s.length(), 
	          new TokenValue(s, yyline, yychar, yychar + s.length()))); }

<YYINITIAL> {ALPHA}+ {
        String str = yytext();
        int sym = Utility.findKeyWord(str);
	return (new Symbol(sym, yychar, yychar+yytext().length(),
	       new TokenValue(yytext(), yyline, yychar, 
		   yychar + yytext().length()))); }

<YYINITIAL> {ALPHA}({ALPHA}|{DIGIT}|_)* {
	return (new Symbol(sym.T_IDENTIFER, yychar, yychar+yytext().length(),
			   new TokenValue(yytext(), yyline, yychar, 
					yychar + yytext().length()))); }
<YYINITIAL> . {
        System.out.println("Illegal character: <" + yytext() + ">");
	Utility.error(Utility.E_UNMATCHED); }




