package imp.slicer;
import imp.parser.Exp;
import imp.parser.Loc;
import imp.util.Util;
import imp.util.VarIdSet;

public class IfThenElseTestNode extends IfThenTestNode {
	public Node elseSuccessorNode;
	
    public IfThenElseTestNode(Loc l, Exp test, int i, TestEndNode endNode, 
    							Node thenSucc, Node elseSucc) {
      super(l, test, i, endNode, thenSucc);
      this.elseSuccessorNode = elseSucc;
    }
    
    public String toString() {
      String locStr = loc.toString();
      String expStr = test.toString();
      String indStr = Util.makeIndentStr(indentLevel);
      return locStr+indStr+"IF "+expStr+" THEN";
    }
    
    public String toString(Node stopNode) {
    	if (this != stopNode) {
    		// convert test node
    		String str = this.toString() + "\n";
    		// recursively convert then-branch
    		str = str + (this.thenSuccessorNode.toString(this.testEndNode));
    		String indStr = Util.makeIndentStr(this.indentLevel);
			str = str + indStr + "ELSE\n";
    		// recursively convert else-branch
    		str = str + (this.elseSuccessorNode.toString(this.testEndNode));
    		// convert end node
    		str = str + (this.testEndNode.toString()) + "\n";
    		if (stopNode != testEndNode) {
    			Node nodeAfterIfThenElse = this.testEndNode.getNext();
    			str = str + nodeAfterIfThenElse.toString(stopNode);
    		}	
    		return str;
    	}
    	else 
    		return "";
    }
    
    /** 
     * Creates string representation of this node if it is relevant (by looking
     * at the "isRelevant" flag). Otherwise, it returns "". 
     * Overwrites method in Node class.
     */
    public String toStringUsingRelevant(Node stopNode) {
    	if (this != stopNode) {
    		String str = "";
    		if (this.isRelevant) {
    			str = this.toString() + "\n";
    			Node nextNode = this.thenSuccessorNode;
    			str = str + nextNode.toStringUsingRelevant(this.testEndNode);
    			String indStr = Util.makeIndentStr(this.indentLevel);
    			str = str + indStr + "ELSE\n";	
    			nextNode = this.elseSuccessorNode;
        		// recursively convert else-branch
        		str = str + (nextNode.toStringUsingRelevant(this.testEndNode));
        		// convert end node
        		Node nodeAfterTestEndNode = this.testEndNode.getNext();
    			if (nodeAfterTestEndNode instanceof EndNode || 
    					nodeAfterTestEndNode instanceof RepeatTestNode)
    				str = str + this.testEndNode.toString() + "\n";
    			else 
    				str = str + this.testEndNode.toString() + ";\n";
    		}
    		// is there more we need to do?
    		if (this.testEndNode != stopNode) {
    			Node nodeAfterTestEndNode = this.testEndNode.getNext();
    			str = str + (nodeAfterTestEndNode.toStringUsingRelevant(stopNode));
    			return str;
    		}
    		else return str;
    	}
    	else 
    		return "";
  	}
    
    /** Fill in this method
     */
    public void computeDRVars(Node cNode, VarIdSet cVars) {
    }
} // public class TestNode extends Node 
