package imp.slicer;

import imp.parser.Exp;
import imp.parser.Loc;
import imp.util.Util;
import imp.util.VarIdSet;

/*
 * A RepeatTestNode has exactly two predecessors: the repeat node and the last 
 * node in the body of the repeat.
 */
public class RepeatTestNode extends TestNode {
	Node firstBodyNode; 
	
    public RepeatTestNode(Loc l, Exp test, int i, Node bodyNode) {
    	super(l, test, i);
	    this.firstBodyNode = bodyNode;
    }
    
    public String toString() {
    	String locStr = loc.toString();
    	String expStr = test.toString();
    	String indStr = Util.makeIndentStr(indentLevel);
    	Node firstNodeAfterRepeatTest = this.getFirstNodeOutside();
    	if (firstNodeAfterRepeatTest instanceof EndNode || 
    			firstNodeAfterRepeatTest instanceof RepeatTestNode) 
    		return locStr+indStr+"UNTIL "+expStr+"\n";
    	else 
    		return locStr+indStr+"UNTIL "+expStr+";\n";
    }
    
    /**
     * Returns the first node that follows the repeat loop, that is,
     * the successor of this node that is not the RepeatNode.
     */
    public Node getFirstNodeOutside() {
    	Node n0 = (Node) this.nexts.elementAt(0);
    	if (n0 != firstBodyNode) 
    		return n0;
    	else 
    		return (Node) this.nexts.elementAt(1);    	
    }
    
    public String toString(Node stopNode) {
    	if (this != stopNode) {
    		String str = this.toString();
    		Node nodeAfterRepeat = this.getFirstNodeOutside();
    		str = str + nodeAfterRepeat.toString(stopNode);
    		return str;
    	}
    	else
    		return "";
    }
 
    /** 
     * Creates string representation of this node if it is relevant (by looking
     * at the "isRelevant" flag). Otherwise, it returns "". 
     * Overwrites method in Node class.
     */
    public String toStringUsingRelevant(Node stopNode) {
    	if (this != stopNode) {
    		String str="";
    		if (this.isRelevant) {
    			str = this.toString();
    			Node nodeAfterRepeat = this.getFirstNodeOutside();
    			str = str + nodeAfterRepeat.toString(stopNode);
    		}
    		return str;
    	}
    	else
    		return "";
    }
 
    /** Fill in this method
     */
    public void computeDRVars(Node cNode, VarIdSet cVars) {
    }
    
} // public class RepeatTestNode extends Node 
