package imp.slicer;

import imp.parser.Exp;
import imp.parser.Loc;
import imp.util.Util;
import imp.util.VarIdSet;

public class WhileTestNode extends TestNode {
	public Node bodySuccessorNode;
	public TestEndNode testEndNode;
	
    public WhileTestNode(Loc l, Exp test, int i, 
    						TestEndNode endNode, Node bodySucc) {
    	super(l, test, i);
	    this.bodySuccessorNode = bodySucc;
	    this.testEndNode = endNode;
    }
    
    public String toString() {
	      String locStr = loc.toString();
	      String expStr = test.toString();
	      String indStr = Util.makeIndentStr(indentLevel);
	      return locStr+indStr+"WHILE "+expStr+" DO";
    }
    
    public String toString(Node stopNode) {
    	if (this != stopNode) {
    		// convert test node
    		String str = this.toString() + "\n";
    		// recursively convert everything in the body till the test node
    		str = str + (this.bodySuccessorNode).toString(this);
    		// convert end node
    		str = str + (this.testEndNode.toString()) + "\n";
    		// is there more?
    		if (stopNode != testEndNode) {
    			Node nodeAfterWhile = testEndNode.getNext();
    			str = str + (nodeAfterWhile.toString(stopNode));
    		}
    		return str;
    	}
    	else 
    		return "";
    }
    
    /** 
     * Creates string representation of this node if it is relevant (by looking
     * at the "isRelevant" flag). Otherwise, it returns "". 
     * Overwrites method in Node class.
     */
    public String toStringUsingRelevant(Node stopNode) {
    	if (this != stopNode) {
    		// convert test node
    		String str = "";
    		if (this.isRelevant) {
    			str = this.toString() + "\n";
    			// recursively convert everything in the body till the test node
    			str = str + (this.bodySuccessorNode).toStringUsingRelevant(this);
    			// convert end node
        		Node nodeAfterTestEndNode = this.testEndNode.getNext();
    			if (nodeAfterTestEndNode instanceof EndNode ||
    					nodeAfterTestEndNode instanceof RepeatTestNode) 
    				str = str + this.testEndNode.toString() + "\n";
    			else 
    				str = str + this.testEndNode.toString() + ";\n";
    		}
    		// is there more?
    		if (this.testEndNode != stopNode) {
    			Node nodeAfterWhile = testEndNode.getNext();
    			str = str + (nodeAfterWhile.toStringUsingRelevant(stopNode));
    		}
    		return str;
    	}
    	else 
    		return "";
    }
     
    /** Fill in this method
     */
    public void computeDRVars(Node cNode, VarIdSet cVars) {
    }
    
} // public class TestNode extends Node 
